@@ -12,17 +12,17 @@ use Carp;
 
 sub get_pdls {my($this) = @_; return ($this->{ParNames},$this->{ParObjs});}
 
-my @code_args_always = qw(SignatureObj GenericTypes ExtraGenericSwitches HaveThreading Name);
+my @code_args_always = qw(BadFlag SignatureObj GenericTypes ExtraGenericSwitches HaveBroadcasting Name);
 sub make_args {
-  my ($which) = @_;
-  ("Parsed$which", [$which,\"Bad$which",@code_args_always]);
+  my ($target) = @_;
+  ("${target}CodeParsed", ["${target}CodeUnparsed",\"Bad${target}CodeUnparsed",@code_args_always]);
 }
 
 # Do the appropriate substitutions in the code.
 sub new {
     my($class,$code,$badcode,
-       $sig,$generictypes,$extrageneric,$havethreading,$name,
-       $dont_add_thrloop, $backcode ) = @_;
+       $handlebad, $sig,$generictypes,$extrageneric,$havebroadcasting,$name,
+       $dont_add_brcloop, $backcode, $nulldatacheck) = @_;
     my $parnames = $sig->names_sorted;
 
     die "Error: missing name argument to PDL::PP::Code->new call!\n"
@@ -30,15 +30,14 @@ sub new {
     confess "Error: empty or undefined GenericTypes!\n"
       unless @{$generictypes || []};
 
-    # simple way of handling bad code check
-    my $handlebad = defined($badcode);
+    $badcode //= $code if $handlebad;
 
     # last two arguments may not be supplied
     #
-    # "backcode" is a flag to the PDL::PP::Threadloop class indicating thre threadloop
+    # "backcode" is a flag to the PDL::PP::Broadcastloop class indicating the broadcastloop
     #   is for writeback code (typically used for writeback of data from child to parent PDL
 
-    $dont_add_thrloop ||= !$havethreading; # two have identical (though inverted) meaning so only track one
+    $dont_add_brcloop ||= !$havebroadcasting; # two have identical (though inverted) meaning so only track one
 
     # C++ style comments
     #
@@ -49,7 +48,7 @@ sub new {
 
     if ($::PP_VERBOSE) {
 	print "Processing code for $name\n";
-	print "DONT_ADD_THRLOOP!\n" if $dont_add_thrloop;
+	print "DONT_ADD_BRCLOOP!\n" if $dont_add_brcloop;
 	print "EXTRAGEN: {" .
 	  join(" ",
 	       map "$_=>$$extrageneric{$_}", sort keys %$extrageneric)
@@ -68,22 +67,23 @@ sub new {
 	ftypes_type => undef,
         Generictypes => $generictypes,   # so that MacroAccess can check it
         Name => $name,
+        NullDataCheck => $nulldatacheck,
     }, $class;
 
     # First, separate the code into an array of C fragments (strings),
     # variable references (strings starting with $) and
     # loops (array references, 1. item = variable.
     #
-    my ( $threadloops, $coderef, $sizeprivs ) =
-	$this->separate_code( "{$code\n}" );
+    my ( $broadcastloops, $coderef, $sizeprivs ) =
+	$this->separate_code( "{\n$code\n}" );
 
-    # Now, if there is no explicit threadlooping in the code,
+    # Now, if there is no explicit broadcastlooping in the code,
     # enclose everything into it.
-    if(!$threadloops && !$dont_add_thrloop) {
-	print "Adding threadloop...\n" if $::PP_VERBOSE;
+    if(!$broadcastloops && !$dont_add_brcloop) {
+	print "Adding broadcastloop...\n" if $::PP_VERBOSE;
 	my $nc = $coderef;
 	$coderef = $backcode
-	  ? PDL::PP::BackCodeThreadLoop->new() : PDL::PP::ThreadLoop->new();
+	  ? PDL::PP::BackCodeBroadcastLoop->new() : PDL::PP::BroadcastLoop->new();
 	push @{$coderef},$nc;
     }
 
@@ -93,19 +93,19 @@ sub new {
     #
     # NOTE: amalgamate sizeprivs from good and bad code
     #
-    if ( $handlebad ) {
+    if ( $handlebad && ($code ne $badcode || $badcode =~ /PDL_BAD_CODE|PDL_IF_BAD/) ) {
 	print "Processing 'bad' code...\n" if $::PP_VERBOSE;
-	my ( $bad_threadloops, $bad_coderef, $bad_sizeprivs ) =
-	    $this->separate_code( "{$badcode\n}" );
+	my ( $bad_broadcastloops, $bad_coderef, $bad_sizeprivs ) =
+	    $this->separate_code( "{\n$badcode\n}" );
 
-	if(!$bad_threadloops && !$dont_add_thrloop) {
-	    print "Adding 'bad' threadloop...\n" if $::PP_VERBOSE;
+	if(!$bad_broadcastloops && !$dont_add_brcloop) {
+	    print "Adding 'bad' broadcastloop...\n" if $::PP_VERBOSE;
 	    my $nc = $bad_coderef;
-	    if( !$backcode ){ # Normal readbackdata threadloop
-		    $bad_coderef = PDL::PP::ThreadLoop->new();
+	    if( !$backcode ){ # Normal readbackdata broadcastloop
+		    $bad_coderef = PDL::PP::BroadcastLoop->new();
 	    }
-	    else{  # writebackcode threadloop
-		    $bad_coderef = PDL::PP::BackCodeThreadLoop->new();
+	    else{  # writebackcode broadcastloop
+		    $bad_coderef = PDL::PP::BackCodeBroadcastLoop->new();
 	    }
 	    push @{$bad_coderef},$nc;
 	}
@@ -151,16 +151,14 @@ sub new {
     # Then, in this form, put it together what we want the code to actually do.
     print "SIZEPRIVS: ",(join ',',%$sizeprivs),"\n" if $::PP_VERBOSE;
     $this->{Code} = (join '',sort values %$sizeprivs).
-       ($dont_add_thrloop?'':PDL::PP::pp_line_numbers __LINE__, join "\n",
-        'PDL_COMMENT("threadloop declarations")',
-        'int __thrloopval;',
+       ($dont_add_brcloop?'':PDL::PP::pp_line_numbers __LINE__, join "\n",
+        'PDL_COMMENT("broadcastloop declarations")',
+        'int __brcloopval;',
         'register PDL_Indx __tind0,__tind1; PDL_COMMENT("counters along dim")',
-        'register PDL_Indx __tnpdls = $PRIV(pdlthread).npdls;',
+        'register PDL_Indx __tnpdls = $PRIV(broadcast).npdls;',
         'PDL_COMMENT("dims here are how many steps along those dims")',
-        (map "register PDL_Indx __tinc0_$parnames->[$_] = PDL_THR_INC(\$PRIV(pdlthread).incs,__tnpdls,$_,0);", 0..$#$parnames),
-        (map "register PDL_Indx __tinc1_$parnames->[$_] = PDL_THR_INC(\$PRIV(pdlthread).incs,__tnpdls,$_,1);", 0..$#$parnames),
-        $this->threadloop_start,
-        $this->threadloop_end,
+        (map "register PDL_Indx __tinc0_$parnames->[$_] = PDL_BRC_INC(\$PRIV(broadcast).incs,__tnpdls,$_,0);", 0..$#$parnames),
+        (map "register PDL_Indx __tinc1_$parnames->[$_] = PDL_BRC_INC(\$PRIV(broadcast).incs,__tnpdls,$_,1);", 0..$#$parnames),
        ).
        $this->params_declare.
        join('',map $_->get_incregisters, @$pobjs{sort keys %$pobjs}).
@@ -173,68 +171,42 @@ sub new {
 sub params_declare {
     my ($this) = @_;
     my ($ord,$pdls) = $this->get_pdls;
-    my @decls = map $_->get_xsdatapdecl("PDL_PARAMTYPE_".$_->name),
+    my @decls = map $_->get_xsdatapdecl("PDL_PARAMTYPE_".$_->name, $this->{NullDataCheck}),
       map $pdls->{$_}, @$ord;
     my @param_names = map "PDL_PARAMTYPE_$_", @$ord;
     PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-#define PDL_DECLARE_PARAMS_$this->{Name}(@{[join ',', @param_names]}) \\
+#ifndef PDL_DECLARE_PARAMS_$this->{Name}_$this->{NullDataCheck}
+#define PDL_DECLARE_PARAMS_$this->{Name}_$this->{NullDataCheck}(@{[join ',', @param_names]}) \\
   @{[join " \\\n", @decls]}
+#endif
 EOF
 }
 
 sub func_name { $_[1] ? "writebackdata" : "readdata" }
-sub threadloop_start_name { "PDL_STARTTHREADLOOP_$_[0]{Name}" }
-sub threadloop_end_name { "PDL_ENDTHREADLOOP_$_[0]->{Name}" }
 
-sub threadloop_start {
-    my ($this) = @_;
+sub broadcastloop_start {
+    my ($this, $funcname) = @_;
     my ($ord,$pdls) = $this->get_pdls;
-    my $macro_name = $this->threadloop_start_name;
-    PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-#define $macro_name(funcName) \\
-__thrloopval = PDL->startthreadloop(&(\$PRIV(pdlthread)),\$PRIV(vtable)->funcName, __privtrans, &PDL_err); \\
-if (PDL_err.error) return PDL_err; \\
-if ( __thrloopval < 0 ) return PDL->make_error_simple(PDL_EFATAL, "Error starting threadloop"); \\
-if ( __thrloopval ) return PDL_err; \\
-       do { \\
-	    PDL_Indx *__tdims = PDL->get_threaddims(&\$PRIV(pdlthread)); \\
-	    if (!__tdims) return PDL->make_error_simple(PDL_EFATAL, "Error in get_threaddims"); \\
-	    register PDL_Indx __tdims1 = __tdims[1]; \\
-	    register PDL_Indx __tdims0 = __tdims[0]; \\
-	    register PDL_Indx *__offsp = PDL->get_threadoffsp(&\$PRIV(pdlthread)); \\
-	    if (!__offsp ) return PDL->make_error_simple(PDL_EFATAL, "Error in get_threadoffsp"); \\
-      PDL_COMMENT("incs are each pdl's stride, declared at func start") \\
-      PDL_COMMENT("offs are each pthread's starting offset into each pdl") \\
-@{[ join " \\\n", map $pdls->{$ord->[$_]}->do_pointeraccess." += __offsp[$_];", 0..$#$ord ]} \\
-      for( __tind1 = 0 ; \\
-	    __tind1 < __tdims1 ; \\
-	    __tind1++ \\
-	    PDL_COMMENT("step by tinc1, undoing inner-loop of tinc0*tdims0") \\
-@{[ join " \\\n", map "\t\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc1_$ord->[$_] - __tinc0_$ord->[$_] * __tdims0", 0..$#$ord ]} \\
-	 ) \\
-      { \\
-	 for( __tind0 = 0 ; \\
-	      __tind0 < __tdims0 ; \\
-	      __tind0++ \\
-@{[ join " \\\n", map "\t\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc0_$ord->[$_]", 0..$#{$ord} ]} \\
-	   ) { \\
-      PDL_COMMENT("This is the tightest threadloop. Make sure inside is optimal.")
+    <<EOF;
+PDL_BROADCASTLOOP_START(
+$funcname,
+\$PRIV(broadcast),
+\$PRIV(vtable),
+@{[ join "", map "\t".$pdls->{$ord->[$_]}->do_pointeraccess." += __offsp[$_];\n", 0..$#$ord ]},
+(@{[ join "", map "\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc1_$ord->[$_] - __tinc0_$ord->[$_] * __tdims0\n", 0..$#$ord ]}),
+(@{[ join "", map "\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc0_$ord->[$_]\n", 0..$#{$ord} ]})
+)
 EOF
 }
 
-sub threadloop_end {
+sub broadcastloop_end {
     my ($this) = @_;
     my ($ord,$pdls) = $this->get_pdls();
-    my $macro_name = $this->threadloop_end_name;
-    PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-#define $macro_name \\
-} \\
-} \\
-PDL_COMMENT("undo outer-loop of tinc1*tdims1, and original per-pthread offset") \\
-@{[ join " \\\n", map $pdls->{$ord->[$_]}->do_pointeraccess." -= __tinc1_$ord->[$_] * __tdims1 + __offsp[$_];", 0..$#$ord ]} \\
-__thrloopval = PDL->iterthreadloop(&\$PRIV(pdlthread),2); \\
-if ( __thrloopval < 0 ) return PDL->make_error_simple(PDL_EFATAL, "Error in iterthreadloop"); \\
-} while(__thrloopval);
+    <<EOF;
+PDL_BROADCASTLOOP_END(
+\$PRIV(broadcast),
+@{[ join "", map $pdls->{$ord->[$_]}->do_pointeraccess." -= __tinc1_$ord->[$_] * __tdims1 + __offsp[$_];\n", 0..$#$ord ]}
+)
 EOF
 }
 
@@ -258,7 +230,7 @@ my %access2class = (
 );
 
 sub process {
-    my ($this, $code, $stack_ref, $threadloops_ref, $sizeprivs) = @_;
+    my ($this, $code, $stack_ref, $broadcastloops_ref, $sizeprivs) = @_;
     while($code) {
 	# Parse next statement
 	$code =~ s/^(.*?) # First, some noise is allowed. This may be bad.
@@ -266,7 +238,7 @@ sub process {
 	        |\$[a-zA-Z_]\w*\s*\([^)]*\)  # $a(...): access
 		|\bloop\s*\([^)]+\)\s*%\{   # loop(..) %{
 		|\btypes\s*\([^)]+\)\s*%\{  # types(..) %{
-		|\bthreadloop\s*%\{         # threadloop %{
+		|\b(?:thread|broadcast)loop\s*%\{         # broadcastloop %{
 		|%}                        # %}
 		|$)//xs
 		    or confess("Invalid program $code");
@@ -285,11 +257,11 @@ sub process {
 	    my $ob = PDL::PP::Types->new($1,$this);
 	    push @{$stack_ref->[-1]},$ob;
 	    push @$stack_ref,$ob;
-	} elsif($control =~ /^threadloop\s*%\{/) {
-	    my $ob = PDL::PP::ThreadLoop->new;
+	} elsif($control =~ /^(?:thread|broadcast)loop\s*%\{/) {
+	    my $ob = PDL::PP::BroadcastLoop->new;
 	    push @{$stack_ref->[-1]},$ob;
 	    push @$stack_ref,$ob;
-	    $$threadloops_ref++;
+	    $$broadcastloops_ref++;
 	} elsif($control =~ /^%}/) {
 	    pop @$stack_ref;
 	} else {
@@ -300,7 +272,7 @@ sub process {
     } # while: $code
 }
 
-# my ( $threadloops, $coderef, $sizeprivs ) = $this->separate_code( $code );
+# my ( $broadcastloops, $coderef, $sizeprivs ) = $this->separate_code( $code );
 #
 # separates the code into an array of C fragments (strings),
 # variable references (strings starting with $) and
@@ -311,15 +283,16 @@ sub separate_code {
     # First check for standard code errors:
     catch_code_errors($code);
     my @stack = my $coderef = PDL::PP::Block->new;
-    my $threadloops = 0;
+    my $broadcastloops = 0;
     my $sizeprivs = {};
-    $this->process($code, \@stack, \$threadloops, $sizeprivs);
-    ( $threadloops, $coderef, $sizeprivs );
+    $this->process($code, \@stack, \$broadcastloops, $sizeprivs);
+    ( $broadcastloops, $coderef, $sizeprivs );
 } # sub: separate_code()
 
+my $macro_pat = qr/\w+/;
 sub expand {
     my ($this, $text) = @_;
-    my (undef, $pdl, $inds, $rest) = PDL::PP::Rule::Substitute::macro_extract($text);
+    my (undef, $pdl, $inds, $rest) = PDL::PP::Rule::Substitute::macro_extract($text, $macro_pat);
     my @add;
     if($pdl =~ /^T/) {@add = PDL::PP::MacroAccess->new($pdl,$inds,
 			   $this->{Generictypes},$this->{Name});}
@@ -332,18 +305,15 @@ sub expand {
 	    $name = $1;
 	    $inds = substr $inds, 1, -1; # chop off brackets
 	} elsif ($get eq 'PP') {
-	    ($name, $inds) = split /\s*,\s*/, $inds;
+	    ($name, $inds) = PDL::PP::Rule::Substitute::split_cpp($inds);
 	} else {
-	    ($inds, $name) = $inds =~ /(.*)\s*,\s*(\w+)/;
+	    ($inds, $name) = PDL::PP::Rule::Substitute::split_cpp($inds);
 	}
 	@add = PDL::PP::BadAccess->new($opcode,$get,$name,$inds,$this);
     }
     elsif($this->{ParObjs}{$pdl}) {@add = PDL::PP::Access->new($pdl,$inds)}
     else {
-	@add = "\$$pdl(";
-	# assumption: the only "control" that will happen in macro args is another macro
-	$this->process($inds, [\@add], undef, undef);
-	push @add, ")";
+	confess "unknown construct $pdl($inds)";
     }
     ($rest, @add);
 }
@@ -447,10 +417,14 @@ sub get_str {
     my $str = PDL::PP::pp_line_numbers(__LINE__, <<EOF);
 if ( \$PRIV(bvalflag) ) { PDL_COMMENT("** do 'bad' Code **")
 #define PDL_BAD_CODE
+#define PDL_IF_BAD(t,f) t
   @{[ $bad->get_str($parent,$context) ]}
 #undef PDL_BAD_CODE
+#undef PDL_IF_BAD
 } else { PDL_COMMENT("** else do 'good' Code **")
+#define PDL_IF_BAD(t,f) f
   @{[ $good->get_str($parent,$context) ]}
+#undef PDL_IF_BAD
 }
 EOF
 }
@@ -518,6 +492,7 @@ sub myprelude {
     qq[PDL_COMMENT("Start generic loop")\n\tswitch($this->[3]) {\n];
 }
 
+my @GENTYPE_ATTRS = qw(integer real unsigned);
 sub myitemstart {
     my ($this,$parent,$nth) = @_;
     my $item = $this->[0][$nth] || return "";
@@ -526,18 +501,25 @@ sub myitemstart {
     my ($ord,$pdls) = $parent->get_pdls;
     my @param_ctypes = map $pdls->{$_}->adjusted_type($item)->ctype, @$ord;
     my $decls = keys %{$this->[2]} == @$ord
-      ? PDL::PP::pp_line_numbers(__LINE__-1, "\t\tPDL_DECLARE_PARAMS_$parent->{Name}(@{[join ',', @param_ctypes]})\n")
-      : join '', map $_->get_xsdatapdecl($_->adjusted_type($item)->ctype),
+      ? PDL::PP::pp_line_numbers(__LINE__-1, "\t\tPDL_DECLARE_PARAMS_$parent->{Name}_$parent->{NullDataCheck}(@{[join ',', @param_ctypes]})\n")
+      : join '', map $_->get_xsdatapdecl($_->adjusted_type($item)->ctype, $parent->{NullDataCheck}),
           map $parent->{ParObjs}{$_}, sort keys %{$this->[2]};
+    my @gentype_decls = map "#define PDL_IF_GENTYPE_".uc($_)."(t,f) ".
+	($item->$_ ? 't' : 'f')."\n",
+	@GENTYPE_ATTRS;
     join '',
 	PDL::PP::pp_line_numbers(__LINE__-1, "case @{[$item->sym]}: {\n"),
+	@gentype_decls,
 	$decls;
 }
 
 sub myitemend {
     my ($this,$parent,$nth) = @_;
     my $item = $this->[0][$nth] || return "";
-    PDL::PP::pp_line_numbers(__LINE__-1, "} break;\n");
+    join '',
+	"\n",
+	(map "#undef PDL_IF_GENTYPE_".uc($_)."\n", @GENTYPE_ATTRS),
+	PDL::PP::pp_line_numbers(__LINE__-1, "} break;\n");
 }
 
 sub mypostlude {
@@ -550,10 +532,10 @@ sub mypostlude {
 
 ####
 #
-# This relies on PP.pm making sure that initthreadstruct always sets
+# This relies on PP.pm making sure that initbroadcaststruct always sets
 # up the two first dimensions even when they are not necessary.
 #
-package PDL::PP::ThreadLoop;
+package PDL::PP::BroadcastLoop;
 use Carp;
 our @ISA = "PDL::PP::Block";
 
@@ -564,20 +546,19 @@ sub new {
 sub myoffs { return 0; }
 sub myprelude {
     my($this,$parent,$context, $backcode) = @_;
-    $parent->threadloop_start_name.'('.$parent->func_name($backcode).')';
+    $parent->broadcastloop_start($parent->func_name($backcode));
 }
 
-# Should possibly fold out thread.dims[0] and [1].
 sub mypostlude {my($this,$parent,$context) = @_;
-    $parent->threadloop_end_name;
+    $parent->broadcastloop_end;
 }
 
-# Simple subclass of ThreadLoop to implement writeback code
+# Simple subclass of BroadcastLoop to implement writeback code
 #
 #
-package PDL::PP::BackCodeThreadLoop;
+package PDL::PP::BackCodeBroadcastLoop;
 use Carp;
-our @ISA = "PDL::PP::ThreadLoop";
+our @ISA = "PDL::PP::BroadcastLoop";
 
 sub myprelude {
     my($this,$parent,$context, $backcode) = @_;
@@ -619,7 +600,7 @@ sub get_str {
 package PDL::PP::Access;
 use Carp;
 
-sub new { my($type,$pdl,$inds,$parent) = @_;
+sub new { my($type,$pdl,$inds) = @_;
     bless [$pdl,$inds],$type;
 }
 
@@ -636,8 +617,8 @@ use Carp;
 
 sub new {
     my ( $type, $opcode, $get, $name, $inds, $parent ) = @_;
-    die "\nIt looks like you have tried a \$${opcode}() macro on an\n" .
-	"  unknown ndarray <$name($inds)>\n"
+    die "\nIt looks like you have tried a $get \$${opcode}() macro on an" .
+	" unknown ndarray <$name($inds)>\n"
 	unless defined($parent->{ParObjs}{$name});
     bless [$opcode, $get, $name, $inds], $type;
 }
@@ -684,8 +665,8 @@ sub new {
     $pdl =~ /^\s*T([A-Z]+)\s*$/
       or confess("Macroaccess wrong in $name (allowed types $types): was '$pdl'\n");
     my @ilst = split '', $1;
-    my @lst = split ',', $inds, -1;
-    confess "Macroaccess: different nos of args $pdl $inds\n" if @lst != @ilst;
+    my @lst = PDL::PP::Rule::Substitute::split_cpp($inds);
+    confess "Macroaccess: different nos of args $pdl (@{[scalar @lst]}=@lst) vs (@{[scalar @ilst]}=@ilst)\n" if @lst != @ilst;
     my %type2value; @type2value{@ilst} = @lst;
     confess "$name has no Macro for generic type $_ (has $pdl)\n"
 	for grep !exists $type2value{$_}, @$gentypes;
