@@ -4,8 +4,8 @@ PDL::Doc::Perldl - commands for accessing PDL doc database from 'perldl' shell
 
 =head1 DESCRIPTION
 
-This module provides a simple set of functions to
-access the PDL documentation of database, for use
+This module provides a set of functions to
+access the PDL documentation database, for use
 from the I<perldl> or I<pdl2> shells as well as the
 I<pdldoc> command-line program.
 
@@ -14,15 +14,24 @@ tree.  That behavior can be switched off with the variable
 C<$PERLDL::STRICT_DOCS> (true: don't search autoload tree; false: search
 the autoload tree.)
 
-Currently, multiple matches are not handled very well.
+In the interest of brevity, functions that print module names (at the moment
+just L</apropos> and L</usage>) use some shorthand notation for module names.
+Currently-implemented shorthands are
+
+=over 3
+
+=item * P:: (short for PDL::)
+
+=item * P::G:: (short for PDL::Graphics::)
+
+=back
+
+To turn this feature off, set the variable $PERLDL::long_mod_names to a true value.
+The feature is assumed to be on for the purposes of this documentation.
 
 =head1 SYNOPSIS
 
- use PDL::Doc::Perldl; # Load all documenation functions
-
-=head1 BUGS
-
-The description contains the misleading word "simple". 
+ use PDL::Doc::Perldl; # Load all documentation functions
 
 =head1 FUNCTIONS
 
@@ -32,21 +41,18 @@ package PDL::Doc::Perldl;
 
 use Exporter;
 use strict;
-use vars qw(@ISA @EXPORT);
+use warnings;
 
-@ISA = qw(Exporter);
-
-@EXPORT = qw( apropos aproposover usage help sig badinfo whatis );
+our @ISA = qw(Exporter);
+our @EXPORT = qw( apropos aproposover usage help sig badinfo whatis );
 
 use PDL::Doc;
-use IO::File;
+use Pod::Select;
 use Pod::PlainText;
+use Term::ReadKey; #for GetTerminalSize
 
 $PDL::onlinedoc = undef;
-$PDL::onlinedoc = new PDL::Doc (FindStdFile());
-
-use PDL::Config;
-my $bvalflag = $PDL::Config{WITH_BADVAL} || 0;
+$PDL::onlinedoc = PDL::Doc->new(FindStdFile());
 
 # Find std file
 
@@ -73,11 +79,7 @@ sub FindStdFile {
 # machines)
 #
 sub screen_width() {
-    return $ENV{COLUMNS}
-       || (($ENV{TERMCAP} =~ /co#(\d+)/) and $1)
-       || ($^O ne 'MSWin32' and $^O ne 'dos' and 
-	   (`stty -a 2>/dev/null` =~ /columns\s*=?\s*(\d+)/) and $1)
-       || 72;
+    return ( ( GetTerminalSize(\*STDOUT) )[0] // 72);
 }
 
 sub printmatch {
@@ -89,6 +91,22 @@ sub printmatch {
     }
 } # sub: print_match()
 
+
+# given a long module name, return the (perhaps shortened) module name.
+
+sub shortmod {
+  my $module = shift;
+  $module =~ s/::$//;
+  unless ($PERLDL::long_mod_names && $PERLDL::long_mod_names){ # silence warn
+      $module =~ s/^PDL::/P::/;
+      $module =~ s/^P::Graphics::/P::G::/;
+      #additional abbreviation substitutions go here
+  }
+  return $module;
+}
+
+
+
 # return a string containing a formated version of the Ref string
 # for the given matches
 #
@@ -96,28 +114,36 @@ sub format_ref {
   my @match = @_;
   my @text = ();
 
-  my $width = screen_width()-17;
+  #finding the max width before doing the printing means looping through @match an extra time; so be it.
+  my @module_shorthands = map { shortmod($_->[1]) } @match;
+  my $max_mod_length = -1;
+  map {$max_mod_length = length if (length>$max_mod_length) } @module_shorthands;
+
+
+  my $width = screen_width()-17-1-$max_mod_length;
   my $parser = new Pod::PlainText( width => $width, indent => 0, sentence => 0 );
 
-  for my $m (@match) { 
-    my $ref = $m->[1]{Ref} || 
-      ( (defined $m->[1]{CustomFile})
-        ? "[No ref avail. for `".$m->[1]{CustomFile}."']"
+  for my $m (@match) {
+    my $ref = $m->[2]{Ref} ||
+      ( (defined $m->[2]{CustomFile})
+        ? "[No ref avail. for `".$m->[2]{CustomFile}."']"
         : "[No reference available]"
      );
 
+    my $name = $m->[0];
+    my $module = shortmod($m->[1]);
+
     $ref = $parser->interpolate( $ref );
     $ref = $parser->reformat( $ref );
 
     # remove last new lines (so substitution doesn't append spaces at end of text)
     $ref =~ s/\n*$//;
-    $ref =~ s/\n/\n                /g;
-
-    my $name = $m->[0];
-    if ( length($name) > 15 ) { 
-      push @text, sprintf "%s ...\n                %s\n", $name, $ref;
+    $ref =~ s/\n/"\n                ".' 'x($max_mod_length+2)/eg;
+    $ref =~ s/^\s*//;
+    if ( length($name) > 15 ) {
+      push @text, sprintf "%s ...\n " . ' 'x15 . "%-*s  %s\n", $name, $max_mod_length, $module, $ref;
     } else {
-      push @text, sprintf "%-15s %s\n", $name, $ref;
+      push @text, sprintf "%-15s %-*s  %s\n", $name, $max_mod_length, $module, $ref;
     }
   }
   return wantarray ? @text : $text[0];
@@ -137,11 +163,15 @@ Regex search PDL documentation database
 =for example
 
  pdl> apropos 'pic'
- rpic            Read images in many formats with automatic format detection.
- rpiccan         Test which image formats can be read/written
- wmpeg           Write an image sequence ((x,y,n) piddle) as an MPEG animation.
- wpic            Write images in many formats with automatic format selection.
- wpiccan         Test which image formats can be read/written
+ PDL::IO::Pic    P::IO::Pic  Module: image I/O for PDL
+ grabpic3d       P::G::TriD  Grab a 3D image from the screen.
+ rim             P::IO::Pic  Read images in most formats, with improved RGB handling.
+ rpic            P::IO::Pic  Read images in many formats with automatic format detection.
+ rpiccan         P::IO::Pic  Test which image formats can be read/written
+ wim             P::IO::Pic  Write a pdl to an image file with selected type (or using filename extensions)
+ wmpeg           P::IO::Pic  Write an image sequence (a (3,x,y,n) byte pdl) as an animation.
+ wpic            P::IO::Pic  Write images in many formats with automatic format selection.
+ wpiccan         P::IO::Pic  Test which image formats can be read/written
 
 To find all the manuals that come with PDL, try
 
@@ -162,7 +192,7 @@ sub aproposover {
     my $func = shift;
     $func =~ s:\/:\\\/:g;
     search_docs("m/$func/",['Name','Ref','Module'],1);
-    
+
 }
 
 sub apropos  {
@@ -186,7 +216,7 @@ sub search_docs {
 
     @match = $PDL::onlinedoc->search($func,$types,$sortflag);
     push(@match,find_autodoc( $func, $exact ) );
-    
+
     @match;
 }
 
@@ -210,11 +240,11 @@ sub finddoc  {
     # See if it matches a PDL function name
 
     my $subfield = $1
-      if( $topic =~ s/\[(\d*)\]$// );
+      if( $topic =~ s/\[(\d*)\]$// ); #does it end with a number in square brackets?
 
-    (my $t2 = $topic) =~ s/([^a-zA-Z0-9_])/\\$1/g;  
+    (my $t2 = $topic) =~ s/([^a-zA-Z0-9_])/\\$1/g;  #$t2 is a copy of $topic with escaped non-word characters
 
-    my @match = search_docs("m/^(PDL::)?".$t2."\$/",['Name'],0);
+    my @match = search_docs("m/^(PDL::)?".$t2."\$/",['Name'],0) ; #matches: ^PDL::topic$ or ^topic$
 
     unless(@match) {
       
@@ -226,7 +256,7 @@ sub finddoc  {
 
     # print out the matches
 
-    my $out = IO::File->new( "| pod2text | $PDL::Doc::pager" );
+    open my $out, "| pod2text | $PDL::Doc::pager";
     
     if($subfield) {
       if($subfield <= @match) {
@@ -244,11 +274,11 @@ sub finddoc  {
     while (@match) {
        $pdl_pod_matchnum++;
 
-       if (  @match > 1   and   !$subfield  ) {
+       if (  @match > 1   and   !$subfield  and $pdl_pod_matchnum==1 ) {
           print $out "\n\n=head1 MULTIPLE MATCHES FOR HELP TOPIC '$topic':\n\n=head1\n\n=over 3\n\n";
           my $i=0;
           for my $m ( @match ) {
-             printf $out "\n=item [%d]\t%-30s %s%s\n\n", ++$i, $m->[0], $m->[1]{Module} && "in ", $m->[1]{CustomFile} || $m->[1]{Module};
+             printf $out "\n=item [%d]\t%-30s %s%s\n\n", ++$i, $m->[0], $m->[2]{Module} && "in ", $m->[2]{CustomFile} || $m->[2]{Module};
           }
           print $out "\n=back\n\n=head1\n\n To see item number \$n, use 'help ${topic}\[\$n\]'. \n\n=cut\n\n";
        }
@@ -259,22 +289,35 @@ sub finddoc  {
 
        my $m = shift @match;
 
-       my $Ref = $m->[1]{Ref};
-       if ( $Ref =~ /^(Module|Manual|Script): / ) {
-          my $in = IO::File->new("<$m->[1]{File}");
-          print $out join("",<$in>);
+       my $Ref = $m->[2]{Ref};
+       if ( $Ref && $Ref =~ /^(Module|Manual|Script): / ) {
+	   # We've got a file name and we have to open it.  With the relocatable db, we have to reconstitute the absolute pathname.
+	   my $relfile = $m->[2]{File};
+	   my $absfile = undef;
+	   my @scnd = @{$PDL::onlinedoc->{Scanned}};
+	   for my $dbf(@scnd){
+	       $dbf =~ s:\/[^\/]*$::; # Trim file name off the end of the database file to get just the directory
+	       $dbf .= "/$relfile";
+	       $absfile = $dbf if( -e $dbf );
+	   }
+	   unless ($absfile) {
+	       die "Documentation error: couldn't find absolute path to $relfile\n";
+	   }
+	   open my $in, "<", $absfile;
+	   print $out join("",<$in>);
        } else {
-          if(defined $m->[1]{CustomFile}) {
+          if(defined $m->[2]{CustomFile}) {
 
-             my $parser= new PDL::Pod::Parser;
-             print $out "=head1 Autoload file \"".$m->[1]{CustomFile}."\"\n\n";
-             $parser->parse_from_file($m->[1]{CustomFile},$out);
-             print $out "\n\n=head2 Docs from\n\n".$m->[1]{CustomFile}."\n\n";
+             my $parser= Pod::Select->new;
+             print $out "=head1 Autoload file \"".$m->[2]{CustomFile}."\"\n\n";
+             $parser->parse_from_file($m->[2]{CustomFile},$out);
+             print $out "\n\n=head2 Docs from\n\n".$m->[2]{CustomFile}."\n\n";
 
           } else {
 
-             print $out "=head1 Module ",$m->[1]{Module}, "\n\n";
-             $PDL::onlinedoc->funcdocs($m->[0],$out);
+             print $out "=head1 Module ",$m->[2]{Module}, "\n\n";
+#	     print STDERR "calling funcdocs(" . $m->[0] . ", " . $m->[1] . ")\n";
+             $PDL::onlinedoc->funcdocs($m->[0],$m->[1],$out);
 
           }
 
@@ -287,10 +330,12 @@ sub finddoc  {
 
 =for ref
 
-Internal helper routine that finds and returns documentation in the autoloader
-path, if it exists.  You feed in a topic and it searches for the file
-"${topic}.pdl".  If that exists, then the filename gets returned in a 
-match structure appropriate for the rest of finddoc.
+Internal routine that finds and returns documentation in the
+PDL::AutoLoader path, if it exists.
+
+You feed in a topic and it searches for the file "${topic}.pdl".  If
+that exists, then the filename gets returned in a match structure
+appropriate for the rest of finddoc.
 
 =cut
 
@@ -326,7 +371,7 @@ sub find_autodoc {
 	if($exact) {
 	    my $file = $dir . "/" . "$topic";
 	    push(@out,
-	          [$file, {CustomFile => "$file", Module => "file '$file'"}]
+	          [$file, undef, {CustomFile => "$file", Module => "file '$file'"}]
 		 )
 		if(-e $file);
 	} else {
@@ -335,7 +380,7 @@ sub find_autodoc {
 	    closedir(FOO);
 	    for my $file( grep( &$matcher, @dir ) ) {
 		push(@out,
-		     [$file, {CustomFile => "$dir/$file", Module => "file '$dir/$file'"}]
+		     [$file, undef, {CustomFile => "$dir/$file", Module => "file '$dir/$file'"}]
 		     );
 	    }
 
@@ -359,10 +404,10 @@ Prints usage information for a PDL function
 
    pdl> usage 'inner'
 
-   inner           inner prodcuct over one dimension
-                   (Module PDL::Primitive)
+   inner           P::Primitive  Inner product over one dimension
+
+   Signature: inner(a(n); b(n); [o]c())
 
-   Signature: inner(a(n); b(n); [o]c(); )
 
 
 =cut
@@ -375,20 +420,24 @@ sub usage {
 sub usage_string{
     my $func = shift;
     my $str = "";
-    my @match = search_docs("m/^(PDL::)?$func\$/",['Name']);
-
-    unless (@match) { $str = "\n  no match\n" } 
+    my @match = search_docs("m/^(PDL::)?$func\$|\:\:$func\$/",['Name']);
+    my $count = @match;
+    unless ($count) { $str = "\n  no match\n" }
     else {
-	$str .= "\n" . format_ref( $match[0] );
-	my ($name,$hash) = @{$match[0]};
-	$str .= sprintf ( (' 'x16)."(Module %s)\n\n", $hash->{Module} );
-	die "No usage info found for $func\n"
-	    if !defined $hash->{Example} && !defined $hash->{Sig} &&
-		!defined $hash->{Usage};
-	$str .= "  Signature: $name($hash->{Sig})\n\n" if defined $hash->{Sig};
-	for (['Usage','Usage'],['Opt','Options'],['Example','Example']) {
-	    $str .= "  $_->[1]:\n\n".&allindent($hash->{$_->[0]},10)."\n\n"
-		if defined $hash->{$_->[0]};
+	#this sorts by namespace depth by counting colons in the name.
+	#PDL::Ufunc::max comes before PDL::GSL::RNG::max, for example.
+	foreach my $m(sort { scalar(()=$a->[1]=~/\:/g) <=> scalar(()=$b->[1]=~/\:/g) } @match){
+	    $str .= "\n" . format_ref( $m );
+	    my ($name,$module,$hash) = @{$m};
+	    #$str .= sprintf ( (' 'x16)."(Module %s)\n\n", $hash->{Module} );
+	    $str.="\n";
+	    die "No usage info found for $func\n" if (!defined $hash->{Example} && !defined $hash->{Sig} &&
+						      !defined $hash->{Usage});
+	    $str .= "  Signature: $name($hash->{Sig})\n\n" if defined $hash->{Sig};
+	    for (['Usage','Usage'],['Opt','Options'],['Example','Example']) {
+		$str .= "  $_->[1]:\n".&allindent($hash->{$_->[0]},10)."\n" if defined $hash->{$_->[0]};
+	    }
+	    $str .= '='x20 unless 1==$count--;
 	}
     }
     return $str;
@@ -406,13 +455,12 @@ prints signature of PDL function
 
 The signature is the normal dimensionality of the
 function's arguments.  Calling with different dimensions
-doesn't break -- it causes threading.  See L<PDL::PP|PDL::PP> for details.
+doesn't break -- it causes broadcasting.  See L<PDL::PP> and L<PDL::Broadcasting> for details.
 
 =for example
 
   pdl> sig 'outer'
-    Signature: outer(a(n); b(m); [o]c(n,m); )
-
+    Signature: outer(a(n); b(m); [o]c(n,m))
 
 =cut
 
@@ -420,13 +468,16 @@ sub sig {
 	die "Usage: sig \$funcname\n" unless $#_>-1;
 	die "no online doc database" unless defined $PDL::onlinedoc;
 	my $func = shift;
-	my @match = search_docs("m/^(PDL::)?$func\$/",['Name']);
+	my @match = search_docs("m/^(PDL::)?$func\$|\:\:$func\$/",['Name']);
+	my $count = @match;
 	unless (@match) { print "\n  no match\n" } else {
-         my ($name,$hash) = @{$match[0]};
-	 die "No signature info found for $func\n"
-            if !defined $hash->{Sig};
-         print "  Signature: $name($hash->{Sig})\n" if defined $hash->{Sig};
-        }
+	    foreach my $m(sort { scalar(()=$a->[1]=~/\:/g) <=> scalar(()=$b->[1]=~/\:/g) } @match){
+		my ($name,$module,$hash) = @{$m};
+		die "No signature info found for $func\n" if !defined $hash->{Sig};
+		print "  Signature: $name($hash->{Sig})\n" if defined $hash->{Sig};
+		print '='x20 unless 1==$count--;
+	    }
+	}
 }
 
 sub allindent {
@@ -478,74 +529,74 @@ $PDL::Doc::Perldl::hash_indent=3;
 sub whatis_r {
   my $prefix = shift;
   my $indent = shift;
-  my $a = shift;
+  my $x = shift;
   
-  unless(defined $a) {
+  unless(defined $x) {
     print $prefix,"<undef>\n";
     return;
   }
 
-  unless(ref $a) {
+  unless(ref $x) {
     print "${prefix}'".
-      substr($a,0,$PDL::Doc::Perldl::max_strlen).
-      "'".((length $a > $PDL::Doc::Perldl::max_strlen) && '...').
+      substr($x,0,$PDL::Doc::Perldl::max_strlen).
+      "'".((length $x > $PDL::Doc::Perldl::max_strlen) && '...').
       "\n";
     return;
   }
 
-  if(ref $a eq 'ARRAY') {
-    print "${prefix}Array (".scalar(@$a)." elements):\n";
+  if(ref $x eq 'ARRAY') {
+    print "${prefix}Array (".scalar(@$x)." elements):\n";
 
     my($el);
-    for $el(0..$#$a) {
+    for $el(0..$#$x) {
       my $pre = sprintf("%s  %2d: "," "x$indent,$el);
-      whatis_r($pre,$indent + $PDL::Doc::Perldl::array_indent, $a->[$el]);
+      whatis_r($pre,$indent + $PDL::Doc::Perldl::array_indent, $x->[$el]);
       last if($el == $PDL::Doc::Perldl::max_arraylen);
     } 
     printf "%s   ... \n"," " x $indent
-      if($#$a > $PDL::Doc::Perldl::max_arraylen);
+      if($#$x > $PDL::Doc::Perldl::max_arraylen);
 
     return;
   }
       
-  if(ref $a eq 'HASH') {
-    print "${prefix}Hash (".scalar(keys %$a)." elements)\n";
+  if(ref $x eq 'HASH') {
+    print "${prefix}Hash (".scalar(keys %$x)." elements)\n";
     my $key;
-    for $key(sort keys %$a) {
+    for $key(sort keys %$x) {
       my $pre = " " x $indent .
 	        " $key: " . 
 		(" "x($PDL::Doc::Perldl::max_keylen - length($key))) ;
 
-      whatis_r($pre,$indent + $PDL::Doc::Perldl::hash_indent, $a->{$key});
+      whatis_r($pre,$indent + $PDL::Doc::Perldl::hash_indent, $x->{$key});
     }
     return;
   }
 
-  if(ref $a eq 'CODE') {
+  if(ref $x eq 'CODE') {
     print "${prefix}Perl CODE ref\n";
     return;
   }
 
-  if(ref $a eq 'SCALAR' | ref $a eq 'REF') {
-    whatis_r($prefix." Ref -> ",$indent+8,$$a);
+  if(ref $x eq 'SCALAR' || ref $x eq 'REF') {
+    whatis_r($prefix." Ref -> ",$indent+8,$$x);
     return;
   }
 
-  if(UNIVERSAL::can($a,'px')) {
-    my $b;
+  if(UNIVERSAL::can($x,'px')) {
+    my $y;
     local $PDL::debug = 1;
 
-    $b = ( (UNIVERSAL::isa($a,'PDL') && $a->nelem < 5 && $a->ndims < 2)
+    $y = ( (UNIVERSAL::isa($x,'PDL') && $x->nelem < 5 && $x->ndims < 2)
 	   ? 
-	   ": $a" :
+	   ": $x" :
 	   ": *****"
 	   );
 
-    $a->px($prefix.(ref $a)." %7T (%D) ".$b);
+    $x->px($prefix.(ref $x)." %7T (%D) ".$y);
 
   } else {
 
-    print "${prefix}Object: ".ref($a)."\n";
+    print "${prefix}Object: ".ref($x)."\n";
 
   }
 }
@@ -575,9 +626,9 @@ and the remaining commands listed, along with the names of their modules.
 sub help_url {
     local $_;
     foreach(@INC) {
-	my $a = "$_/PDL/HtmlDocs/PDL/Index.html";
-	if(-e $a) {
-	    return "file://$a";
+	my $x = "$_/PDL/HtmlDocs/PDL/Index.html";
+	if(-e $x) {
+	    return "file://$x";
 	}
     }
 }
@@ -594,9 +645,9 @@ sub help {
 	  if ($topic =~ /^\s*vars\s*$/i) {
 	      PDL->px((caller)[0]);
 	  } elsif($topic =~ /^\s*url\s*/i) {
-	      my $a = help_url();
-	      if($a) {
-		  print $a;
+	      my $x = help_url();
+	      if($x) {
+		  print $x;
 	      } else {
 		  print "Hmmm. Curious: I couldn't find the HTML docs anywhere in \@INC...\n";
 	      }
@@ -625,22 +676,17 @@ sub help {
 The following commands support online help in the perldl shell:
 
  help 'thing'   -- print docs on 'thing' (func, module, manual, autoload-file)
- help vars      -- print information about all current piddles
+ help vars      -- print information about all current ndarrays
  help url       -- locate the HTML version of the documentation
  help www       -- View docs with default web browser (set by env: PERLDL_WWW)
 
- whatis <expr>  -- Describe the type and structure of an expression or piddle.
+ whatis <expr>  -- Describe the type and structure of an expression or ndarray.
  apropos 'word' -- search for keywords/function names 
  usage          -- print usage information for a given PDL function
  sig            -- print signature of PDL function
+ badinfo        -- information on the support for bad values
 
  ('?' is an alias for 'help';  '??' is an alias for 'apropos'.)
-EOH
-
-print "  badinfo         -- information on the support for bad values\n"
-   if $bvalflag;
-
-print <<'EOH';
 
 Quick start:
   apropos 'manual:' -- Find all the manual documents
@@ -664,6 +710,14 @@ And has a horrible name.
 
  badinfo 'func'
 
+=for example
+
+  pdl> badinfo 'inner'
+  Bad value support for inner (in module PDL::Primitive)
+      If "a() * b()" contains only bad data, "c()" is set bad. Otherwise "c()"
+      will have its bad flag cleared, as it will not contain any bad values.
+
+
 =cut
 
 # need to get this to format the output - want a format_bad()
@@ -673,24 +727,30 @@ sub badinfo {
     my $func = shift;
     die "Usage: badinfo \$funcname\n" unless defined $func;
 
-    die "PDL has not been compiled with support for bad values.\n" .
-	"Recompile with WITH_BADVAL set to 1 in config file!.\n"
-	    unless $bvalflag;
-
     die "no online doc database" unless defined $PDL::onlinedoc;
 
     local $SIG{PIPE}= sub {}; # Prevent crashing if user exits the pager
 
-    my @match = search_docs("m/^(PDL::)?$func\$/",['Name']);
-    if ( @match ) {
-	my ($name,$hash) = @{$match[0]};
-	my $info = $hash->{Bad};
+    my @match = search_docs("m/^(PDL::)?$func\$|\:\:$func\$/",['Name']);
+    my $count = @match;
+    if ( $count ) {
+	my ($pagerstr, $noinfostr) = ('', '');
+	foreach my $m(@match) {
+	    my ($name,$module,$hash) = @{$m};
+	    my $info = $hash->{Bad};
+	    if ( defined $info ) {
+		$name=~s/^(.*)\:\:(\w*)$/$2/;
 
-	if ( defined $info ) {
-	    my $out = new IO::File "| pod2text | $PDL::Doc::pager";
-	    print $out "=head1 Bad value support for $name\n\n$info\n";
+		$pagerstr .= "=head1 Bad value support for $name (in module $module)\n\n$info\n";
+	    } else {
+		$noinfostr .= "\n  No information on bad-value support found for $func (in module $module)\n";
+	    }
+	}
+	if ($pagerstr){
+	    open my $out, "| pod2text | $PDL::Doc::pager";
+	    print $out $pagerstr, $noinfostr;
 	} else {
-	    print "\n  No information on bad-value support found for $func\n";
+	    print $noinfostr;
 	}
     } else {
 	print "\n  no match\n";
